'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.adjustCalendar = exports.compareDates = exports.setToStartOfDay = exports.getDaysInMonth = exports.isWeekend = undefined;

var _lodash = require('lodash.isdate');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Returns true if date is a weekend day, false if not
 * @param  {date} date - Date to check
 * @return {boolean}
 */
var isWeekend = exports.isWeekend = function isWeekend(date) {
    var day = date.getDay();

    return day === 0 || day === 6;
};

/**
 * Returns true if year is a leap year, false if not
 * @param  {number} year - Year to check
 * @return {boolean}
 *
 * Solution by Matti Virkkunen: http://stackoverflow.com/a/4881951
 */
var isLeapYear = function isLeapYear(year) {
    return year % 4 === 0 && year % 100 !== 0 || year % 400 === 0;
};

/**
 * Returns the number of days in a month accounting for leap years
 * @param  {number} year - Year to return number of days for
 * @param  {number} month - Month to return number of days for
 * @return {number}
 */
var getDaysInMonth = exports.getDaysInMonth = function getDaysInMonth(year, month) {
    return [31, isLeapYear(year) ? 29 : 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31][month];
};

/**
 * Sets a date's time to 0, 0, 0, 0 for comparing dates
 * @param  {date} date - Date to set time on
 */
var setToStartOfDay = exports.setToStartOfDay = function setToStartOfDay(date) {
    if ((0, _lodash2.default)(date)) date.setHours(0, 0, 0, 0);
};

/**
 * Weak date comparison (use setToStartOfDay(date) to ensure correct result)
 * @param  {date} a - Date to compare
 * @param  {date} b - Date to compare
 * @return {boolean}
 */
var compareDates = exports.compareDates = function compareDates(a, b) {
    return a.getTime() === b.getTime();
};

/**
 * Month/year wrapping
 * @param  {object} - Calendar to adjust
 * @return {object}
 */
var adjustCalendar = exports.adjustCalendar = function adjustCalendar(calendar) {
    if (calendar.month < 0) {
        calendar.year -= Math.ceil(Math.abs(calendar.month) / 12);
        calendar.month += 12;
    }
    if (calendar.month > 11) {
        calendar.year += Math.floor(Math.abs(calendar.month) / 12);
        calendar.month -= 12;
    }
    return calendar;
};