'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.fireEvent = exports.removeEvent = exports.addEvent = exports.hasEventListeners = undefined;

var _lodash = require('lodash.assign');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var hasEventListeners = exports.hasEventListeners = !!window.addEventListener;

/**
 * @param  {DOM element} el - DOM element to add event to
 * @param  {string} e - Event to add
 * @param  {function} callback - Callback
 * @param  {boolean} capture - Whether to capture the event
 */
var addEvent = exports.addEvent = function addEvent(el, e, callback, capture) {
    if (hasEventListeners) {
        el.addEventListener(e, callback, !!capture);
    } else {
        el.attachEvent('on' + e, callback);
    }
};

/**
 * @param  {DOM element} el - DOM element to add event to
 * @param  {string} e - Event to add
 * @param  {function} callback - Callback
 * @param  {boolean} capture - Whether to capture the event
 */
var removeEvent = exports.removeEvent = function removeEvent(el, e, callback, capture) {
    if (hasEventListeners) {
        el.removeEventListener(e, callback, !!capture);
    } else {
        el.detachEvent('on' + e, callback);
    }
};

/**
 * @param  {DOM element} el - DOM element to fire event on
 * @param  {string} eventName - Event to fire
 * @param  {hash} data - Data to fire event with
 */
var fireEvent = exports.fireEvent = function fireEvent(el, eventName, data) {
    var ev = undefined;

    if (window.document.createEvent) {
        ev = window.document.createEvent('HTMLEvents');
        ev.initEvent(eventName, true, false);
        ev = (0, _lodash2.default)(ev, data);
        el.dispatchEvent(ev);
    } else if (window.document.createEventObject) {
        ev = window.document.createEventObject();
        ev = (0, _lodash2.default)(ev, data);
        el.fireEvent('on' + eventName, ev);
    }
};